@@ -10,6 +10,7 @@ __all__ = [
     'events',
     'event',
     'file',
+    'file_request',
     'file_version',
     'file_version_retention',
     'folder',
@@ -38,5 +39,5 @@ __all__ = [
     'upload_session',
     'webhook',
     'watermark',
-    'web_link'
+    'web_link',
 ]
