@@ -6,14 +6,13 @@ from ..util.api_call_decorator import api_call
 
 class Comment(BaseObject):
     """An object that represents a comment on an item"""
+
     _item_type = 'comment'
 
     @staticmethod
     def construct_params_from_message(message: str) -> dict:
         message_type = 'tagged_message' if '@[' in message else 'message'
-        return {
-            message_type: message
-        }
+        return {message_type: message}
 
     @api_call
     def reply(self, message: str) -> 'Comment':
@@ -25,10 +24,7 @@ class Comment(BaseObject):
         """
         url = self.get_type_url()
         data = self.construct_params_from_message(message)
-        data['item'] = {
-            'type': 'comment',
-            'id': self.object_id
-        }
+        data['item'] = {'type': 'comment', 'id': self.object_id}
         box_response = self._session.post(url, data=json.dumps(data))
         response = box_response.json()
         return self.translator.translate(
