@@ -1,14 +1,18 @@
 import json
 import os
 from datetime import datetime
-from typing import TYPE_CHECKING, Optional, Tuple, Union, IO, Iterable, List
+from typing import TYPE_CHECKING, Optional, Tuple, Union, IO, Iterable, List, Any
+from boxsdk.exception import BoxAPIException
 
 from boxsdk.util.datetime_formatter import normalize_date_to_rfc3339_format
 from .item import Item
 from ..util.api_call_decorator import api_call
+from ..util.default_arg_value import SDK_VALUE_NOT_SET
 from ..util.deprecation_decorator import deprecated
 from ..pagination.marker_based_object_collection import MarkerBasedObjectCollection
-from ..pagination.limit_offset_based_object_collection import LimitOffsetBasedObjectCollection
+from ..pagination.limit_offset_based_object_collection import (
+    LimitOffsetBasedObjectCollection,
+)
 
 if TYPE_CHECKING:
     from boxsdk.object.upload_session import UploadSession
@@ -47,7 +51,12 @@ class File(Item):
         )
 
     @api_call
-    def create_upload_session(self, file_size: int, file_name: Optional[str] = None) -> 'UploadSession':
+    def create_upload_session(
+        self,
+        file_size: int,
+        file_name: Optional[str] = None,
+        use_upload_session_urls: bool = True,
+    ) -> 'UploadSession':
         """
         Create a new chunked upload session for uploading a new version of the file.
 
@@ -55,6 +64,11 @@ class File(Item):
             The size of the file in bytes that will be uploaded.
         :param file_name:
             The new name of the file version that will be uploaded.
+        :param use_upload_session_urls:
+            The parameter detrermining what urls to use to perform chunked upload.
+            If True, the urls returned by create_upload_session() endpoint response will be used,
+            unless a custom API.UPLOAD_URL was set in the config.
+            If False, the base upload url will be used.
         :returns:
             A :class:`UploadSession` object.
         """
@@ -64,15 +78,25 @@ class File(Item):
         }
         if file_name is not None:
             body_params['file_name'] = file_name
-        url = self.get_url('upload_sessions').replace(self.session.api_config.BASE_API_URL, self.session.api_config.UPLOAD_URL)
+        url = self.get_url('upload_sessions').replace(
+            self.session.api_config.BASE_API_URL, self.session.api_config.UPLOAD_URL
+        )
         response = self._session.post(url, data=json.dumps(body_params)).json()
-        return self.translator.translate(
+        upload_session = self.translator.translate(
             session=self._session,
             response_object=response,
         )
+        # pylint:disable=protected-access
+        upload_session._use_upload_session_urls = use_upload_session_urls
+        return upload_session
 
     @api_call
-    def get_chunked_uploader(self, file_path: str, rename_file: bool = False) -> 'ChunkedUploader':
+    def get_chunked_uploader(
+        self,
+        file_path: str,
+        rename_file: bool = False,
+        use_upload_session_urls: bool = True,
+    ) -> 'ChunkedUploader':
         # pylint: disable=consider-using-with
         """
         Instantiate the chunked upload instance and create upload session with path to file.
@@ -81,14 +105,23 @@ class File(Item):
             The local path to the file you wish to upload.
         :param rename_file:
             Indicates whether the file should be renamed or not.
+        :param use_upload_session_urls:
+            The parameter detrermining what urls to use to perform chunked upload.
+            If True, the urls returned by create_upload_session() endpoint response will be used,
+            unless a custom API.UPLOAD_URL was set in the config.
+            If False, the base upload url will be used.
         :returns:
             A :class:`ChunkedUploader` object.
         """
         total_size = os.stat(file_path).st_size
         content_stream = open(file_path, 'rb')
         file_name = os.path.basename(file_path) if rename_file else None
-        upload_session = self.create_upload_session(total_size, file_name)
-        return upload_session.get_chunked_uploader_for_stream(content_stream, total_size)
+        upload_session = self.create_upload_session(
+            total_size, file_name, use_upload_session_urls
+        )
+        return upload_session.get_chunked_uploader_for_stream(
+            content_stream, total_size
+        )
 
     def _get_accelerator_upload_url_for_update(self) -> Optional[str]:
         """
@@ -118,7 +151,11 @@ class File(Item):
         raise ValueError('Expected a 1-tuple or 2-tuple for byte range')
 
     @api_call
-    def content(self, file_version: Optional['FileVersion'] = None, byte_range: Tuple[int, int] = None) -> bytes:
+    def content(
+        self,
+        file_version: Optional['FileVersion'] = None,
+        byte_range: Tuple[int, int] = None,
+    ) -> bytes:
         """
         Get the content of a file on Box.
 
@@ -130,17 +167,25 @@ class File(Item):
             File content as bytes.
         """
         url = self.get_url('content')
-        params = {'version': file_version.object_id} if file_version is not None else None
-        headers = {'Range': self._construct_range_header(byte_range)} if byte_range is not None else None
-        box_response = self._session.get(url, expect_json_response=False, params=params, headers=headers)
+        params = (
+            {'version': file_version.object_id} if file_version is not None else None
+        )
+        headers = (
+            {'Range': self._construct_range_header(byte_range)}
+            if byte_range is not None
+            else None
+        )
+        box_response = self._session.get(
+            url, expect_json_response=False, params=params, headers=headers
+        )
         return box_response.content
 
     @api_call
     def download_to(
-            self,
-            writeable_stream: IO[bytes],
-            file_version: Optional['FileVersion'] = None,
-            byte_range: Tuple[int, int] = None
+        self,
+        writeable_stream: IO[bytes],
+        file_version: Optional['FileVersion'] = None,
+        byte_range: Tuple[int, int] = None,
     ) -> None:
         """
         Download the file; write it to the given stream.
@@ -153,10 +198,20 @@ class File(Item):
             A tuple of inclusive byte offsets to download, e.g. (100, 199) to download the second 100 bytes of a file
         """
         url = self.get_url('content')
-        params = {'version': file_version.object_id} if file_version is not None else None
-        headers = {'Range': self._construct_range_header(byte_range)} if byte_range is not None else None
-        box_response = self._session.get(url, expect_json_response=False, stream=True, params=params, headers=headers)
-        for chunk in box_response.network_response.response_as_stream.stream(decode_content=True):
+        params = (
+            {'version': file_version.object_id} if file_version is not None else None
+        )
+        headers = (
+            {'Range': self._construct_range_header(byte_range)}
+            if byte_range is not None
+            else None
+        )
+        box_response = self._session.get(
+            url, expect_json_response=False, stream=True, params=params, headers=headers
+        )
+        for chunk in box_response.network_response.response_as_stream.stream(
+            decode_content=True
+        ):
             writeable_stream.write(chunk)
 
     @api_call
@@ -169,27 +224,40 @@ class File(Item):
         :return: Url to download the file
         """
         url = self.get_url('content')
-        params = {'version': file_version.object_id} if file_version is not None else None
+        params = (
+            {'version': file_version.object_id} if file_version is not None else None
+        )
         box_response = self._session.get(
             url,
             params=params,
             expect_json_response=False,
             allow_redirects=False,
         )
+        network_response = box_response.network_response
+        if 'location' not in box_response.headers:
+            raise BoxAPIException(
+                status=network_response.status_code,
+                headers=network_response.headers,
+                message='Download URL is not present in the response.',
+                url=url,
+                method='GET',
+                network_response=network_response,
+            )
+
         return box_response.headers['location']
 
     @api_call
     def update_contents_with_stream(
-            self,
-            file_stream: IO[bytes],
-            etag: Optional[str] = None,
-            preflight_check: bool = False,
-            preflight_expected_size: int = 0,
-            upload_using_accelerator: bool = False,
-            file_name: Optional[str] = None,
-            content_modified_at: Optional[str] = None,
-            additional_attributes: Optional[dict] = None,
-            sha1: Optional[str] = None,
+        self,
+        file_stream: IO[bytes],
+        etag: Optional[str] = None,
+        preflight_check: bool = False,
+        preflight_expected_size: int = 0,
+        upload_using_accelerator: bool = False,
+        file_name: Optional[str] = None,
+        content_modified_at: Union[datetime, str] = None,
+        additional_attributes: Optional[dict] = None,
+        sha1: Optional[str] = None,
     ) -> 'File':
         """
         Upload a new version of a file, taking the contents from the given file stream.
@@ -213,7 +281,9 @@ class File(Item):
         :param file_name:
             The new name to give the file on Box.
         :param content_modified_at:
-            The RFC-3339 datetime when the file content was last modified.
+            The A datetime string in a format supported by the dateutil library or datetime object,
+            which specifies when the file content was last modified.
+            If no timezone info provided, local timezone will be applied.
         :param additional_attributes:
             A dictionary containing attributes to add to the file that are not covered by other parameters.
         :param sha1:
@@ -240,7 +310,9 @@ class File(Item):
 
         attributes = {
             'name': file_name,
-            'content_modified_at': content_modified_at,
+            'content_modified_at': normalize_date_to_rfc3339_format(
+                content_modified_at
+            ),
         }
         if additional_attributes:
             attributes.update(additional_attributes)
@@ -271,16 +343,16 @@ class File(Item):
 
     @api_call
     def update_contents(
-            self,
-            file_path: str,
-            etag: Optional[str] = None,
-            preflight_check: bool = False,
-            preflight_expected_size: int = 0,
-            upload_using_accelerator: bool = False,
-            file_name: Optional[str] = None,
-            content_modified_at: Optional[str] = None,
-            additional_attributes: Optional[dict] = None,
-            sha1: Optional[str] = None,
+        self,
+        file_path: str,
+        etag: Optional[str] = None,
+        preflight_check: bool = False,
+        preflight_expected_size: int = 0,
+        upload_using_accelerator: bool = False,
+        file_name: Optional[str] = None,
+        content_modified_at: Union[datetime, str] = None,
+        additional_attributes: Optional[dict] = None,
+        sha1: Optional[str] = None,
     ) -> 'File':
         """Upload a new version of a file. The contents are taken from the given file path.
 
@@ -303,7 +375,9 @@ class File(Item):
         :param file_name:
             The new name to give the file on Box.
         :param content_modified_at:
-            The RFC-3339 datetime when the file content was last modified.
+            A datetime string in a format supported by the dateutil library or a datetime.datetime object,
+            which specifies when the file content was last modified.
+            If no timezone info provided, local timezone will be applied.
         :param additional_attributes:
             A dictionary containing attributes to add to the file that are not covered by other parameters.
         :param sha1:
@@ -328,14 +402,18 @@ class File(Item):
             )
 
     @api_call
-    def lock(self, prevent_download: bool = False, expire_time: Optional[str] = None) -> 'File':
+    def lock(
+        self, prevent_download: bool = False, expire_time: Union[datetime, str] = None
+    ) -> 'File':
         """
         Lock a file, preventing others from modifying (or possibly even downloading) it.
 
         :param prevent_download:
             Whether or not the lock should prevent other users from downloading the file.
         :param expire_time:
-            The RFC-3339 datetime when the lock should automatically expire, unlocking the file.
+            A datetime string in a format supported by the dateutil library or a datetime.datetime object,
+            which specifies when the lock should automatically expire, unlocking the file.
+            If no timezone info provided, local timezone will be applied.
         :return:
             A new :class:`File` instance reflecting that the file has been locked.
         """
@@ -346,7 +424,7 @@ class File(Item):
             }
         }
         if expire_time is not None:
-            data['lock']['expires_at'] = expire_time
+            data['lock']['expires_at'] = normalize_date_to_rfc3339_format(expire_time)
         return self.update_info(data=data)
 
     @api_call
@@ -360,15 +438,136 @@ class File(Item):
         data = {'lock': None}
         return self.update_info(data=data)
 
+    @api_call
+    def create_shared_link(
+        self,
+        *,
+        access: Optional[str] = None,
+        etag: Optional[str] = None,
+        unshared_at: Union[datetime, str, None] = SDK_VALUE_NOT_SET,
+        allow_download: Optional[bool] = None,
+        allow_preview: Optional[bool] = None,
+        allow_edit: Optional[bool] = None,
+        password: Optional[str] = None,
+        vanity_name: Optional[str] = None,
+        **kwargs: Any,
+    ) -> 'File':
+        """
+        Baseclass override.
+
+        :param access:
+            Determines who can access the shared link. May be open, company, or collaborators. If no access is
+            specified, the default access will be used.
+        :param etag:
+            If specified, instruct the Box API to create the link only if the current version's etag matches.
+        :param unshared_at:
+            The date on which this link should be disabled. May only be set if the current user is not a free user
+            and has permission to set expiration dates. Takes a datetime string supported by the dateutil library
+            or a datetime.datetime object. If no timezone info provided, local timezone will be applied.
+            The time portion can be omitted, which defaults to midnight (00:00:00) on that date.
+        :param allow_download:
+            Whether the file being shared can be downloaded when accessed via the shared link.
+            If this parameter is None, the default setting will be used.
+        :param allow_preview:
+            Whether the file being shared can be previewed when accessed via the shared link.
+            If this parameter is None, the default setting will be used.
+        :param allow_edit:
+            Whether the file being shared can be edited when accessed via the shared link.
+            If this parameter is None, the default setting will be used.
+        :param password:
+            The password required to view this link. If no password is specified then no password will be set.
+            Please notice that this is a premium feature, which might not be available to your app.
+        :param vanity_name:
+            Defines a custom vanity name to use in the shared link URL, eg. https://app.box.com/v/my-custom-vanity-name.
+            If this parameter is None, the standard shared link URL will be used.
+        :param kwargs:
+            Used to fulfill the contract of overriden method
+        :return:
+            The updated object with shared link.
+            Returns a new object of the same type, without modifying the original object passed as self.
+        :raises: :class:`BoxAPIException` if the specified etag doesn't match the latest version of the file.
+        """
+        # pylint:disable=arguments-differ
+        return super().create_shared_link(
+            access=access,
+            etag=etag,
+            unshared_at=unshared_at,
+            allow_download=allow_download,
+            allow_preview=allow_preview,
+            allow_edit=allow_edit,
+            password=password,
+            vanity_name=vanity_name,
+        )
+
+    @api_call
+    def get_shared_link(
+        self,
+        *,
+        access: Optional[str] = None,
+        etag: Optional[str] = None,
+        unshared_at: Union[datetime, str, None] = SDK_VALUE_NOT_SET,
+        allow_download: Optional[bool] = None,
+        allow_preview: Optional[bool] = None,
+        allow_edit: Optional[bool] = None,
+        password: Optional[str] = None,
+        vanity_name: Optional[str] = None,
+        **kwargs: Any,
+    ) -> 'str':
+        """
+        Baseclass override.
+
+        :param access:
+            Determines who can access the shared link. May be open, company, or collaborators. If no access is
+            specified, the default access will be used.
+        :param etag:
+            If specified, instruct the Box API to create the link only if the current version's etag matches.
+        :param unshared_at:
+            The date on which this link should be disabled. May only be set if the current user is not a free user
+            and has permission to set expiration dates. Takes a datetime string supported by the dateutil library
+            or a datetime.datetime object. If no timezone info provided, local timezone will be applied.
+            The time portion can be omitted, which defaults to midnight (00:00:00) on that date.
+        :param allow_download:
+            Whether the file being shared can be downloaded when accessed via the shared link.
+            If this parameter is None, the default setting will be used.
+        :param allow_preview:
+            Whether the file being shared can be previewed when accessed via the shared link.
+            If this parameter is None, the default setting will be used.
+        :param allow_edit:
+            Whether the file being shared can be edited when accessed via the shared link.
+            If this parameter is None, the default setting will be used.
+        :param password:
+            The password required to view this link. If no password is specified then no password will be set.
+            Please notice that this is a premium feature, which might not be available to your app.
+        :param vanity_name:
+            Defines a custom vanity name to use in the shared link URL, eg. https://app.box.com/v/my-custom-vanity-name.
+            If this parameter is None, the standard shared link URL will be used.
+        :param kwargs:
+            Used to fulfill the contract of overriden method
+        :returns:
+            The URL of the shared link.
+        :raises: :class:`BoxAPIException` if the specified etag doesn't match the latest version of the file.
+        """
+        # pylint:disable=arguments-differ
+        return super().get_shared_link(
+            access=access,
+            etag=etag,
+            unshared_at=unshared_at,
+            allow_download=allow_download,
+            allow_preview=allow_preview,
+            allow_edit=allow_edit,
+            password=password,
+            vanity_name=vanity_name,
+        )
+
     @api_call
     def get_shared_link_download_url(
-            self,
-            access: Optional[str] = None,
-            etag: Optional[str] = None,
-            unshared_at: Optional[str] = None,
-            allow_preview: Optional[bool] = None,
-            password: Optional[str] = None,
-            vanity_name: Optional[str] = None
+        self,
+        access: Optional[str] = None,
+        etag: Optional[str] = None,
+        unshared_at: Union[datetime, str, None] = SDK_VALUE_NOT_SET,
+        allow_preview: Optional[bool] = None,
+        password: Optional[str] = None,
+        vanity_name: Optional[str] = None,
     ) -> str:
         """
         Get a shared link download url for the file with the given access permissions.
@@ -381,8 +580,8 @@ class File(Item):
             If specified, instruct the Box API to create the link only if the current version's etag matches.
         :param unshared_at:
             The date on which this link should be disabled. May only be set if the current user is not a free user
-            and has permission to set expiration dates.  Takes an RFC3339-formatted string, e.g.
-            '2018-10-31T23:59:59-07:00' for 11:59:59 PM on October 31, 2018 in the America/Los_Angeles timezone.
+            and has permission to set expiration dates. Takes a datetime string supported by the dateutil library
+            or a datetime.datetime object. If no timezone info provided, local timezone will be applied.
             The time portion can be omitted, which defaults to midnight (00:00:00) on that date.
         :param allow_preview:
             Whether or not the item being shared can be previewed when accessed via the shared link.
@@ -403,16 +602,13 @@ class File(Item):
             unshared_at=unshared_at,
             allow_preview=allow_preview,
             password=password,
-            vanity_name=vanity_name
+            vanity_name=vanity_name,
         )
         return item.shared_link['download_url']  # pylint:disable=no-member
 
     @api_call
     def get_comments(
-            self,
-            limit: Optional[int] = None,
-            offset: int = 0,
-            fields: Iterable[str] = None
+        self, limit: Optional[int] = None, offset: int = 0, fields: Iterable[str] = None
     ) -> 'BoxObjectCollection':
         """
         Get the comments on the file.
@@ -447,10 +643,7 @@ class File(Item):
         url = self._session.get_url('comments')
         comment_class = self._session.translator.get('comment')
         data = comment_class.construct_params_from_message(message)
-        data['item'] = {
-            'type': 'file',
-            'id': self.object_id
-        }
+        data['item'] = {'type': 'file', 'id': self.object_id}
         box_response = self._session.post(url, data=json.dumps(data))
         response = box_response.json()
         return self._session.translator.translate(
@@ -460,11 +653,11 @@ class File(Item):
 
     @api_call
     def create_task(
-            self,
-            message: Optional[str] = None,
-            due_at: Optional[str] = None,
-            action: str = 'review',
-            completion_rule: Optional[str] = None
+        self,
+        message: Optional[str] = None,
+        due_at: Union[datetime, str] = None,
+        action: str = 'review',
+        completion_rule: Optional[str] = None,
     ) -> 'Task':
         """
         Create a task on the given file.
@@ -472,7 +665,8 @@ class File(Item):
         :param message:
             An optional message to include in the task.
         :param due_at:
-            When this task is due.
+            When this task is due. Takes a datetime string supported by the dateutil library
+            or a datetime.datetime object. If no timezone info provided, local timezone will be applied
         :param action:
             The type of task the task assignee will be prompted to perform.
             Value is one of review,complete
@@ -485,16 +679,13 @@ class File(Item):
         """
         url = self._session.get_url('tasks')
         task_attributes = {
-            'item': {
-                'type': 'file',
-                'id': self.object_id
-            },
+            'item': {'type': 'file', 'id': self.object_id},
             'action': action,
         }
         if message is not None:
             task_attributes['message'] = message
         if due_at is not None:
-            task_attributes['due_at'] = due_at
+            task_attributes['due_at'] = normalize_date_to_rfc3339_format(due_at)
         if completion_rule is not None:
             task_attributes['completion_rule'] = completion_rule
         box_response = self._session.post(url, data=json.dumps(task_attributes))
@@ -525,10 +716,10 @@ class File(Item):
 
     @api_call
     def get_previous_versions(
-            self,
-            limit: Optional[int] = None,
-            offset: int = None,
-            fields: Iterable[str] = None
+        self,
+        limit: Optional[int] = None,
+        offset: int = None,
+        fields: Iterable[str] = None,
     ) -> 'BoxObjectCollection':
         """
         Get previous versions of the file.
@@ -574,7 +765,9 @@ class File(Item):
         )
 
     @api_call
-    def delete_version(self, file_version: 'FileVersion', etag: Optional[str] = None) -> bool:
+    def delete_version(
+        self, file_version: 'FileVersion', etag: Optional[str] = None
+    ) -> bool:
         """
         Delete a specific version of a file.
 
@@ -587,7 +780,9 @@ class File(Item):
         """
         url = self.get_url('versions', file_version.object_id)
         headers = {'If-Match': etag} if etag is not None else None
-        response = self._session.delete(url, expect_json_response=False, headers=headers)
+        response = self._session.delete(
+            url, expect_json_response=False, headers=headers
+        )
         return response.ok
 
     @api_call
@@ -622,12 +817,12 @@ class File(Item):
     @deprecated('Use get_thumbnail_representation')
     @api_call
     def get_thumbnail(
-            self,
-            extension: str = 'png',
-            min_width: Optional[int] = None,
-            min_height: Optional[int] = None,
-            max_width: Optional[int] = None,
-            max_height: Optional[int] = None
+        self,
+        extension: str = 'png',
+        min_width: Optional[int] = None,
+        min_height: Optional[int] = None,
+        max_width: Optional[int] = None,
+        max_height: Optional[int] = None,
     ) -> bytes:
         """
         Retrieve a thumbnail image for the file.
@@ -660,7 +855,9 @@ class File(Item):
         return response.content
 
     @api_call
-    def get_thumbnail_representation(self, dimensions: str, extension: str = 'png') -> bytes:
+    def get_thumbnail_representation(
+        self, dimensions: str, extension: str = 'png'
+    ) -> bytes:
         """
         Retrieve a thumbnail image for the file.
 
@@ -675,7 +872,10 @@ class File(Item):
         representations = self.get_representation_info(rep_hints)
         if representations:
             representation = representations[0]
-            if representation['status'].get('code') in ('error_conversion_failed', 'error_password_protected'):
+            if representation['status'].get('code') in (
+                'error_conversion_failed',
+                'error_password_protected',
+            ):
                 return b''
             url = representation['content']['url_template']
             url = url.replace('{+asset_path}', '')
@@ -685,12 +885,12 @@ class File(Item):
 
     @api_call
     def copy(
-            self,
-            *,
-            parent_folder: 'Folder',
-            name: Optional[str] = None,
-            file_version: 'FileVersion' = None,
-            **_kwargs
+        self,
+        *,
+        parent_folder: 'Folder',
+        name: Optional[str] = None,
+        file_version: 'FileVersion' = None,
+        **_kwargs,
     ) -> 'File':
         # pylint: disable=arguments-differ
         """Copy the item to the given folder.
@@ -706,9 +906,7 @@ class File(Item):
         """
         # pylint: disable=arguments-differ
         url = self.get_url('copy')
-        data = {
-            'parent': {'id': parent_folder.object_id}
-        }
+        data = {'parent': {'id': parent_folder.object_id}}
         if name is not None:
             data['name'] = name
         if file_version is not None:
@@ -726,8 +924,8 @@ class File(Item):
         Modifies the retention expiration timestamp for the given file. This date can't be shortened once set on a file.
 
         :param date_time:
-            A datetime str, eg. '2012-12-12T10:53:43-08:00' or datetime.datetime object. If no timezone info provided,
-            local timezone will be aplied.
+            A datetime string in a format supported by the dateutil library or a datetime.datetime object.
+            If no timezone info provided, local timezone will be applied.
         :return:
             Updated 'File' object
         """
