@@ -1,13 +1,18 @@
 import json
 import os
+from datetime import datetime
 from typing import TYPE_CHECKING, Any, Tuple, Optional, Iterable, IO, Union
 
 from boxsdk.object.group import Group
 from boxsdk.object.item import Item
 from boxsdk.object.user import User
-from boxsdk.pagination.limit_offset_based_object_collection import LimitOffsetBasedObjectCollection
+from boxsdk.pagination.limit_offset_based_object_collection import (
+    LimitOffsetBasedObjectCollection,
+)
 from boxsdk.pagination.marker_based_object_collection import MarkerBasedObjectCollection
 from boxsdk.util.api_call_decorator import api_call
+from boxsdk.util.datetime_formatter import normalize_date_to_rfc3339_format
+from boxsdk.util.default_arg_value import SDK_VALUE_NOT_SET
 from boxsdk.util.text_enum import TextEnum
 
 if TYPE_CHECKING:
@@ -29,6 +34,7 @@ class FolderSyncState(TextEnum):
     The value of the ``sync_state`` attribute determines whether the folder
     will be synced by sync clients.
     """
+
     IS_SYNCED = 'synced'
     NOT_SYNCED = 'not_synced'
     PARTIALLY_SYNCED = 'partially_synced'
@@ -36,12 +42,14 @@ class FolderSyncState(TextEnum):
 
 class _CollaborationType(TextEnum):
     """The type of a collaboration"""
+
     USER = 'user'
     GROUP = 'group'
 
 
 class _Collaborator:
     """This helper class represents a collaborator on Box. A Collaborator can be a User, Group, or an email address"""
+
     def __init__(self, collaborator: Any):
         if isinstance(collaborator, User):
             self._setup(user=collaborator)
@@ -52,7 +60,9 @@ class _Collaborator:
         else:
             raise TypeError('Collaborator must be User, Group, or unicode string')
 
-    def _setup(self, user: User = None, group: Group = None, email_address: str = None) -> None:
+    def _setup(
+        self, user: User = None, group: Group = None, email_address: str = None
+    ) -> None:
         """
         :param user:
             The Box user if applicable
@@ -111,7 +121,9 @@ class Folder(Item):
         )
 
     @api_call
-    def create_upload_session(self, file_size: int, file_name: str) -> 'UploadSession':
+    def create_upload_session(
+        self, file_size: int, file_name: str, use_upload_session_urls: bool = True
+    ) -> 'UploadSession':
         """
         Creates a new chunked upload session for upload a new file.
 
@@ -119,6 +131,11 @@ class Folder(Item):
             The size of the file in bytes that will be uploaded.
         :param file_name:
             The name of the file that will be uploaded.
+        :param use_upload_session_urls:
+            The parameter detrermining what urls to use to perform chunked upload.
+            If True, the urls returned by create_upload_session() endpoint response will be used,
+            unless a custom API.UPLOAD_URL was set in the config.
+            If False, the base upload url will be used.
         :returns:
             A :class:`UploadSession` object.
         """
@@ -129,27 +146,52 @@ class Folder(Item):
             'file_name': file_name,
         }
         response = self._session.post(url, data=json.dumps(body_params)).json()
-        return self.translator.translate(
+        upload_session = self.translator.translate(
             session=self._session,
             response_object=response,
         )
+        # pylint:disable=protected-access
+        upload_session._use_upload_session_urls = use_upload_session_urls
+        return upload_session
 
     @api_call
-    def get_chunked_uploader(self, file_path: str) -> 'ChunkedUploader':
+    def get_chunked_uploader(
+        self,
+        file_path: str,
+        file_name: Optional[str] = None,
+        use_upload_session_urls: bool = True,
+    ) -> 'ChunkedUploader':
         # pylint: disable=consider-using-with
         """
         Instantiate the chunked upload instance and create upload session with path to file.
 
         :param file_path:
             The local path to the file you wish to upload.
+         :param file_name:
+            The name with extention of the file that will be uploaded, e.g. new_file_name.zip.
+            If not specified, the name from the local system is used.
+        :param use_upload_session_urls:
+            The parameter detrermining what urls to use to perform chunked upload.
+            If True, the urls returned by create_upload_session() endpoint response will be used,
+            unless a custom API.UPLOAD_URL was set in the config.
+            If False, the base upload url will be used.
         :returns:
             A :class:`ChunkedUploader` object.
         """
         total_size = os.stat(file_path).st_size
+        upload_file_name = file_name if file_name else os.path.basename(file_path)
         content_stream = open(file_path, 'rb')
-        file_name = os.path.basename(file_path)
-        upload_session = self.create_upload_session(total_size, file_name)
-        return upload_session.get_chunked_uploader_for_stream(content_stream, total_size)
+
+        try:
+            upload_session = self.create_upload_session(
+                total_size, upload_file_name, use_upload_session_urls
+            )
+            return upload_session.get_chunked_uploader_for_stream(
+                content_stream, total_size
+            )
+        except Exception:
+            content_stream.close()
+            raise
 
     def _get_accelerator_upload_url_fow_new_uploads(self) -> Optional[str]:
         """
@@ -162,14 +204,14 @@ class Folder(Item):
 
     @api_call
     def get_items(
-            self,
-            limit: Optional[int] = None,
-            offset: int = 0,
-            marker: Optional[str] = None,
-            use_marker: bool = False,
-            sort: Optional[str] = None,
-            direction: Optional[str] = None,
-            fields: Iterable[str] = None
+        self,
+        limit: Optional[int] = None,
+        offset: int = 0,
+        marker: Optional[str] = None,
+        use_marker: bool = False,
+        sort: Optional[str] = None,
+        direction: Optional[str] = None,
+        fields: Iterable[str] = None,
     ) -> Iterable[Item]:
         """
         Get the items in a folder.
@@ -224,18 +266,19 @@ class Folder(Item):
 
     @api_call
     def upload_stream(
-            self,
-            file_stream: IO[bytes],
-            file_name: str,
-            file_description: Optional[str] = None,
-            preflight_check: bool = False,
-            preflight_expected_size: int = 0,
-            upload_using_accelerator: bool = False,
-            content_created_at: Optional[str] = None,
-            content_modified_at: Optional[str] = None,
-            additional_attributes: Optional[dict] = None,
-            sha1: Optional[str] = None,
-            etag: Optional[str] = None,
+        self,
+        file_stream: IO[bytes],
+        file_name: str,
+        file_description: Optional[str] = None,
+        preflight_check: bool = False,
+        preflight_expected_size: int = 0,
+        upload_using_accelerator: bool = False,
+        content_created_at: Union[datetime, str] = None,
+        content_modified_at: Union[datetime, str] = None,
+        additional_attributes: Optional[dict] = None,
+        sha1: Optional[str] = None,
+        etag: Optional[str] = None,
+        stream_file_content: bool = True,
     ) -> 'File':
         """
         Upload a file to the folder.
@@ -260,22 +303,29 @@ class Folder(Item):
 
             Please notice that this is a premium feature, which might not be available to your app.
         :param content_created_at:
-            The RFC-3339 datetime when the file was created.
+            A datetime string in a format supported by the dateutil library or a datetime.datetime object,
+            which specifies when the file was created. If no timezone info provided, local timezone will be applied.
         :param content_modified_at:
-            The RFC-3339 datetime when the file content was last modified.
+            A datetime string in a format supported by the dateutil library or a datetime.datetime object, which
+            specifies when the file was last modified. If no timezone info provided, local timezone will be applied.
         :param additional_attributes:
             A dictionary containing attributes to add to the file that are not covered by other parameters.
         :param sha1:
             A sha1 checksum for the file.
         :param etag:
             If specified, instruct the Box API to update the item only if the current version's etag matches.
+        :param stream_file_content:
+            If True, the upload will be performed as a stream request. If False, the file will be read into memory
+            before being uploaded, but this may be required if using some proxy servers to handle redirects correctly.
         :returns:
             The newly uploaded file.
         """
         accelerator_upload_url = None
         if preflight_check:
             # Preflight check does double duty, returning the accelerator URL if one is available in the response.
-            accelerator_upload_url = self.preflight_check(size=preflight_expected_size, name=file_name)
+            accelerator_upload_url = self.preflight_check(
+                size=preflight_expected_size, name=file_name
+            )
         elif upload_using_accelerator:
             accelerator_upload_url = self._get_accelerator_upload_url_fow_new_uploads()
 
@@ -287,8 +337,10 @@ class Folder(Item):
             'name': file_name,
             'parent': {'id': self._object_id},
             'description': file_description,
-            'content_created_at': content_created_at,
-            'content_modified_at': content_modified_at,
+            'content_created_at': normalize_date_to_rfc3339_format(content_created_at),
+            'content_modified_at': normalize_date_to_rfc3339_format(
+                content_modified_at
+            ),
         }
         if additional_attributes:
             attributes.update(additional_attributes)
@@ -305,7 +357,14 @@ class Folder(Item):
             headers['Content-MD5'] = sha1
         if not headers:
             headers = None
-        file_response = self._session.post(url, data=data, files=files, expect_json_response=False, headers=headers).json()
+        file_response = self._session.post(
+            url,
+            data=data,
+            files=files,
+            expect_json_response=False,
+            headers=headers,
+            stream_file_content=stream_file_content,
+        ).json()
         if 'entries' in file_response:
             file_response = file_response['entries'][0]
         return self.translator.translate(
@@ -315,18 +374,19 @@ class Folder(Item):
 
     @api_call
     def upload(
-            self,
-            file_path: str = None,
-            file_name: str = None,
-            file_description: Optional[str] = None,
-            preflight_check: bool = False,
-            preflight_expected_size: int = 0,
-            upload_using_accelerator: bool = False,
-            content_created_at: Optional[str] = None,
-            content_modified_at: Optional[str] = None,
-            additional_attributes: Optional[dict] = None,
-            sha1: Optional[str] = None,
-            etag: Optional[str] = None,
+        self,
+        file_path: str = None,
+        file_name: str = None,
+        file_description: Optional[str] = None,
+        preflight_check: bool = False,
+        preflight_expected_size: int = 0,
+        upload_using_accelerator: bool = False,
+        content_created_at: Union[datetime, str] = None,
+        content_modified_at: Union[datetime, str] = None,
+        additional_attributes: Optional[dict] = None,
+        sha1: Optional[str] = None,
+        etag: Optional[str] = None,
+        stream_file_content: bool = True,
     ) -> 'File':
         """
         Upload a file to the folder.
@@ -352,15 +412,20 @@ class Folder(Item):
 
             Please notice that this is a premium feature, which might not be available to your app.
         :param content_created_at:
-            The RFC-3339 datetime when the file was created.
+            A datetime string in a format supported by the dateutil library or a datetime.datetime object,
+            which specifies when the file was created. If no timezone info provided, local timezone will be applied.
         :param content_modified_at:
-            The RFC-3339 datetime when the file content was last modified.
+            A datetime string in a format supported by the dateutil library or a datetime.datetime object, which
+            specifies when the file was last modified.If no timezone info provided, local timezone will be applied.
         :param additional_attributes:
             A dictionary containing attributes to add to the file that are not covered by other parameters.
         :param sha1:
             A sha1 checksum for the new content.
         :param etag:
             If specified, instruct the Box API to update the item only if the current version's etag matches.
+        :param stream_file_content:
+            If True, the upload will be performed as a stream request. If False, the file will be read into memory
+            before being uploaded, but this may be required if using some proxy servers to handle redirects correctly.
         :returns:
             The newly uploaded file.
         """
@@ -379,6 +444,7 @@ class Folder(Item):
                 additional_attributes=additional_attributes,
                 sha1=sha1,
                 etag=etag,
+                stream_file_content=stream_file_content,
             )
 
     @api_call
@@ -394,7 +460,7 @@ class Folder(Item):
             'name': name,
             'parent': {
                 'id': self._object_id,
-            }
+            },
         }
         box_response = self._session.post(url, data=json.dumps(data))
         response = box_response.json()
@@ -420,18 +486,129 @@ class Folder(Item):
         }
         return self.update_info(data=data)
 
+    @api_call
+    def create_shared_link(
+        self,
+        *,
+        access: Optional[str] = None,
+        etag: Optional[str] = None,
+        unshared_at: Union[datetime, str, None] = SDK_VALUE_NOT_SET,
+        allow_download: Optional[bool] = None,
+        allow_preview: Optional[bool] = None,
+        password: Optional[str] = None,
+        vanity_name: Optional[str] = None,
+        **kwargs: Any,
+    ) -> 'Folder':
+        """
+        Baseclass override.
+
+        :param access:
+            Determines who can access the shared link. May be open, company, or collaborators. If no access is
+            specified, the default access will be used.
+        :param etag:
+            If specified, instruct the Box API to create the link only if the current version's etag matches.
+        :param unshared_at:
+            The date on which this link should be disabled. May only be set if the current user is not a free user
+            and has permission to set expiration dates. Takes a datetime string supported by the dateutil library
+            or a datetime.datetime object. If no timezone info provided, local timezone will be applied.
+            The time portion can be omitted, which defaults to midnight (00:00:00) on that date.
+        :param allow_download:
+            Whether the folder being shared can be downloaded when accessed via the shared link.
+            If this parameter is None, the default setting will be used.
+        :param allow_preview:
+            Whether the folder being shared can be previewed when accessed via the shared link.
+            If this parameter is None, the default setting will be used.
+        :param password:
+            The password required to view this link. If no password is specified then no password will be set.
+            Please notice that this is a premium feature, which might not be available to your app.
+        :param vanity_name:
+            Defines a custom vanity name to use in the shared link URL, eg. https://app.box.com/v/my-custom-vanity-name.
+            If this parameter is None, the standard shared link URL will be used.
+        :param kwargs:
+            Used to fulfill the contract of overriden method
+        :return:
+            The updated object with shared link.
+            Returns a new object of the same type, without modifying the original object passed as self.
+        :raises: :class:`BoxAPIException` if the specified etag doesn't match the latest version of the folder.
+        """
+        # pylint:disable=arguments-differ
+        return super().create_shared_link(
+            access=access,
+            etag=etag,
+            unshared_at=unshared_at,
+            allow_download=allow_download,
+            allow_preview=allow_preview,
+            password=password,
+            vanity_name=vanity_name,
+        )
+
+    @api_call
+    def get_shared_link(
+        self,
+        *,
+        access: Optional[str] = None,
+        etag: Optional[str] = None,
+        unshared_at: Union[datetime, str, None] = SDK_VALUE_NOT_SET,
+        allow_download: Optional[bool] = None,
+        allow_preview: Optional[bool] = None,
+        password: Optional[str] = None,
+        vanity_name: Optional[str] = None,
+        **kwargs: Any,
+    ) -> 'str':
+        """
+        Baseclass override.
+
+        :param access:
+            Determines who can access the shared link. May be open, company, or collaborators. If no access is
+            specified, the default access will be used.
+        :param etag:
+            If specified, instruct the Box API to create the link only if the current version's etag matches.
+        :param unshared_at:
+            The date on which this link should be disabled. May only be set if the current user is not a free user
+            and has permission to set expiration dates. Takes a datetime string supported by the dateutil library
+            or a datetime.datetime object. If no timezone info provided, local timezone will be applied.
+            The time portion can be omitted, which defaults to midnight (00:00:00) on that date.
+        :param allow_download:
+            Whether the folder being shared can be downloaded when accessed via the shared link.
+            If this parameter is None, the default setting will be used.
+        :param allow_preview:
+            Whether the folder being shared can be previewed when accessed via the shared link.
+            If this parameter is None, the default setting will be used.
+        :param password:
+            The password required to view this link. If no password is specified then no password will be set.
+            Please notice that this is a premium feature, which might not be available to your app.
+        :param vanity_name:
+            Defines a custom vanity name to use in the shared link URL, eg. https://app.box.com/v/my-custom-vanity-name.
+            If this parameter is None, the standard shared link URL will be used.
+        :param kwargs:
+            Used to fulfill the contract of overriden method
+        :returns:
+            The URL of the shared link.
+        :raises: :class:`BoxAPIException` if the specified etag doesn't match the latest version of the folder.
+        """
+        # pylint:disable=arguments-differ
+        return super().get_shared_link(
+            access=access,
+            etag=etag,
+            unshared_at=unshared_at,
+            allow_download=allow_download,
+            allow_preview=allow_preview,
+            password=password,
+            vanity_name=vanity_name,
+        )
+
     @api_call
     def add_collaborator(
-            self,
-            collaborator: Union[User, Group, str],
-            role: 'CollaborationRole',
-            notify: bool = False,
-            can_view_path: bool = False
+        self,
+        collaborator: Union[User, Group, str],
+        role: 'CollaborationRole',
+        notify: bool = False,
+        can_view_path: bool = False,
     ) -> 'Collaboration':
         """Add a collaborator to the folder
 
         :param collaborator:
-            collaborator to add. May be a User, Group, or email address (unicode string)
+            collaborator to add. It may be a User, Group, or email address (unicode string)
         :param role:
             The collaboration role
         :param notify:
@@ -459,7 +636,9 @@ class Folder(Item):
             body_params['can_view_path'] = True
         data = json.dumps(body_params)
         params = {'notify': notify}
-        box_response = self._session.post(url, expect_json_response=True, data=data, params=params)
+        box_response = self._session.post(
+            url, expect_json_response=True, data=data, params=params
+        )
         collaboration_response = box_response.json()
         return self.translator.translate(
             session=self._session,
@@ -468,10 +647,10 @@ class Folder(Item):
 
     @api_call
     def create_web_link(
-            self,
-            target_url: str,
-            name: Optional[str] = None,
-            description: Optional[str] = None
+        self,
+        target_url: str,
+        name: Optional[str] = None,
+        description: Optional[str] = None,
     ) -> 'WebLink':
         """
         Create a WebLink with a given url.
@@ -486,29 +665,19 @@ class Folder(Item):
             A :class:`WebLink` object.
         """
         url = self._session.get_url('web_links')
-        web_link_attributes = {
-            'url': target_url,
-            'parent': {
-                'id': self.object_id
-            }
-        }
+        web_link_attributes = {'url': target_url, 'parent': {'id': self.object_id}}
         if name is not None:
             web_link_attributes['name'] = name
         if description is not None:
             web_link_attributes['description'] = description
         response = self._session.post(url, data=json.dumps(web_link_attributes)).json()
         return self.translator.translate(
-            session=self._session,
-            response_object=response
+            session=self._session, response_object=response
         )
 
     @api_call
     def delete(
-            self,
-            *,
-            recursive: bool = True,
-            etag: Optional[str] = None,
-            **kwargs
+        self, *, recursive: bool = True, etag: Optional[str] = None, **kwargs
     ) -> bool:
         """Base class override. Delete the folder.
 
@@ -525,11 +694,11 @@ class Folder(Item):
 
     @api_call
     def get_metadata_cascade_policies(
-            self,
-            owner_enterprise: 'Enterprise' = None,
-            limit: Optional[int] = None,
-            marker: Optional[str] = None,
-            fields: Iterable[str] = None
+        self,
+        owner_enterprise: 'Enterprise' = None,
+        limit: Optional[int] = None,
+        marker: Optional[str] = None,
+        fields: Iterable[str] = None,
     ) -> 'BoxObjectCollection':
         """
         Get the metadata cascade policies current applied to the folder.
@@ -563,7 +732,9 @@ class Folder(Item):
         )
 
     @api_call
-    def cascade_metadata(self, metadata_template: 'MetadataTemplate') -> 'MetadataCascadePolicy':
+    def cascade_metadata(
+        self, metadata_template: 'MetadataTemplate'
+    ) -> 'MetadataCascadePolicy':
         """
         Create a metadata cascade policy to apply the metadata instance values on the folder for the given metadata
         template to all files within the folder.
@@ -595,14 +766,8 @@ class Folder(Item):
         url = self._session.get_url('folder_locks')
 
         body = {
-            'folder': {
-                'type': 'folder',
-                'id': self.object_id
-            },
-            'locked_operations': {
-                'move': True,
-                'delete': True
-            }
+            'folder': {'type': 'folder', 'id': self.object_id},
+            'locked_operations': {'move': True, 'delete': True},
         }
 
         response = self._session.post(url, data=json.dumps(body)).json()
