@@ -6,7 +6,6 @@ from .base_item import BaseItem
 from ..exception import BoxAPIException
 from .metadata import Metadata
 from ..util.api_call_decorator import api_call
-from ..util.default_arg_value import SDK_VALUE_NOT_SET
 from ..pagination.marker_based_dict_collection import MarkerBasedDictCollection
 from ..pagination.marker_based_object_collection import MarkerBasedObjectCollection
 
@@ -20,6 +19,7 @@ if TYPE_CHECKING:
 
 class ClassificationType(TextEnum):
     """An enum of possible classification types"""
+
     PUBLIC = 'Public'
     INTERNAL = 'Internal'
     CONFIDENTIAL = 'Confidential'
@@ -31,7 +31,9 @@ class Item(BaseItem):
 
     _classification_template_key = 'securityClassification-6VMVochwUWo'
 
-    def _get_accelerator_upload_url(self, file_id: Optional[str] = None) -> Optional[str]:
+    def _get_accelerator_upload_url(
+        self, file_id: Optional[str] = None
+    ) -> Optional[str]:
         """
         Make an API call to get the Accelerator upload url for either upload a new file or updating an existing file.
 
@@ -54,10 +56,7 @@ class Item(BaseItem):
             return None
 
     def _preflight_check(
-            self, size: int,
-            name: str = None,
-            file_id: str = None,
-            parent_id: str = None
+        self, size: int, name: str = None, file_id: str = None, parent_id: str = None
     ) -> Optional[str]:
         """
         Make an API call to check if certain file can be uploaded to Box or not.
@@ -97,7 +96,9 @@ class Item(BaseItem):
         return response_json.get('upload_url', None)
 
     @api_call
-    def update_info(self, *, data: dict, etag: Optional[str] = None, **kwargs: Any) -> 'Item':
+    def update_info(
+        self, *, data: dict, etag: Optional[str] = None, **kwargs: Any
+    ) -> 'Item':
         """
         Baseclass override.
         :param data:
@@ -121,7 +122,9 @@ class Item(BaseItem):
         return super().update_info(data=data, headers=headers, **kwargs)
 
     @api_call
-    def get(self, *, fields: Iterable[str] = None, etag: Optional[str] = None, **kwargs) -> 'Item':
+    def get(
+        self, *, fields: Iterable[str] = None, etag: Optional[str] = None, **kwargs
+    ) -> 'Item':
         """
         Base class override.
 
@@ -138,115 +141,6 @@ class Item(BaseItem):
         headers = {'If-None-Match': etag} if etag is not None else None
         return super().get(fields=fields, headers=headers, **kwargs)
 
-    @api_call
-    def create_shared_link(
-            self,
-            *,
-            access: Optional[str] = None,
-            etag: Optional[str] = None,
-            unshared_at: Optional[str] = SDK_VALUE_NOT_SET,
-            allow_download: Optional[bool] = None,
-            allow_preview: Optional[bool] = None,
-            password: Optional[str] = None,
-            vanity_name: Optional[str] = None,
-            **kwargs: Any
-    ) -> 'Item':
-        """
-        Baseclass override.
-
-        :param access:
-            Determines who can access the shared link. May be open, company, or collaborators. If no access is
-            specified, the default access will be used.
-        :param etag:
-            If specified, instruct the Box API to create the link only if the current version's etag matches.
-        :param unshared_at:
-            The date on which this link should be disabled. May only be set if the current user is not a free user
-            and has permission to set expiration dates.  Takes an RFC3339-formatted string, e.g.
-            '2018-10-31T23:59:59-07:00' for 11:59:59 PM on October 31, 2018 in the America/Los_Angeles timezone.
-            The time portion can be omitted, which defaults to midnight (00:00:00) on that date.
-        :param allow_download:
-            Whether or not the item being shared can be downloaded when accessed via the shared link.
-            If this parameter is None, the default setting will be used.
-        :param allow_preview:
-            Whether or not the item being shared can be previewed when accessed via the shared link.
-            If this parameter is None, the default setting will be used.
-        :param password:
-            The password required to view this link. If no password is specified then no password will be set.
-            Please notice that this is a premium feature, which might not be available to your app.
-        :param vanity_name:
-            Defines a custom vanity name to use in the shared link URL, eg. https://app.box.com/v/my-custom-vanity-name.
-            If this parameter is None, the standard shared link URL will be used.
-        :param kwargs:
-            Used to fulfill the contract of overriden method
-        :return:
-            The updated object with shared link.
-            Returns a new object of the same type, without modifying the original object passed as self.
-        :raises: :class:`BoxAPIException` if the specified etag doesn't match the latest version of the item.
-        """
-        # pylint:disable=arguments-differ
-        return super().create_shared_link(
-            access=access,
-            etag=etag,
-            unshared_at=unshared_at,
-            allow_download=allow_download,
-            allow_preview=allow_preview,
-            password=password,
-            vanity_name=vanity_name
-        )
-
-    @api_call
-    def get_shared_link(
-            self,
-            *,
-            access: Optional[str] = None,
-            etag: Optional[str] = None,
-            unshared_at: Optional[str] = SDK_VALUE_NOT_SET,
-            allow_download: Optional[bool] = None,
-            allow_preview: Optional[bool] = None,
-            password: Optional[str] = None,
-            vanity_name: Optional[str] = None,
-            **kwargs: Any
-    ) -> 'str':
-        """
-        Baseclass override.
-
-        :param access:
-            Determines who can access the shared link. May be open, company, or collaborators. If no access is
-            specified, the default access will be used.
-        :param etag:
-            If specified, instruct the Box API to create the link only if the current version's etag matches.
-        :param unshared_at:
-            The date on which this link should be disabled. May only be set if the current user is not a free user
-            and has permission to set expiration dates.
-        :param allow_download:
-            Whether or not the item being shared can be downloaded when accessed via the shared link.
-            If this parameter is None, the default setting will be used.
-        :param allow_preview:
-            Whether or not the item being shared can be previewed when accessed via the shared link.
-            If this parameter is None, the default setting will be used.
-        :param password:
-            The password required to view this link. If no password is specified then no password will be set.
-            Please notice that this is a premium feature, which might not be available to your app.
-        :param vanity_name:
-            Defines a custom vanity name to use in the shared link URL, eg. https://app.box.com/v/my-custom-vanity-name.
-            If this parameter is None, the standard shared link URL will be used.
-        :param kwargs:
-            Used to fulfill the contract of overriden method
-        :returns:
-            The URL of the shared link.
-        :raises: :class:`BoxAPIException` if the specified etag doesn't match the latest version of the item.
-        """
-        # pylint:disable=arguments-differ
-        return super().get_shared_link(
-            access=access,
-            etag=etag,
-            unshared_at=unshared_at,
-            allow_download=allow_download,
-            allow_preview=allow_preview,
-            password=password,
-            vanity_name=vanity_name
-        )
-
     @api_call
     def remove_shared_link(self, *, etag: Optional[str] = None, **kwargs: Any) -> bool:
         """
@@ -264,7 +158,9 @@ class Item(BaseItem):
         return super().remove_shared_link(etag=etag)
 
     @api_call
-    def delete(self, *, params: dict = None, etag: Optional[str] = None, **kwargs) -> bool:
+    def delete(
+        self, *, params: dict = None, etag: Optional[str] = None, **kwargs
+    ) -> bool:
         """Delete the item.
 
         :param params:
@@ -332,11 +228,7 @@ class Item(BaseItem):
         """
         self.validate_item_id(self._object_id)
         url = self.get_url('watermark')
-        body_attributes = {
-            'watermark': {
-                'imprint': 'default'
-            }
-        }
+        body_attributes = {'watermark': {'imprint': 'default'}}
         box_response = self._session.put(url, data=json.dumps(body_attributes))
         response = box_response.json()
         return self.translator.get('watermark')(response['watermark'])
@@ -356,12 +248,12 @@ class Item(BaseItem):
 
     @api_call
     def collaborate(
-            self,
-            accessible_by: Union['User', 'Group'],
-            role: str,
-            can_view_path: Optional[bool] = None,
-            notify: Optional[bool] = None,
-            fields: Iterable[str] = None
+        self,
+        accessible_by: Union['User', 'Group'],
+        role: str,
+        can_view_path: Optional[bool] = None,
+        notify: Optional[bool] = None,
+        fields: Iterable[str] = None,
     ) -> 'Collaboration':
         """Collaborate user or group onto a Box item.
 
@@ -407,12 +299,12 @@ class Item(BaseItem):
 
     @api_call
     def collaborate_with_login(
-            self,
-            login: str,
-            role: str,
-            can_view_path: Optional[bool] = None,
-            notify: Optional[bool] = None,
-            fields: Iterable[str] = None
+        self,
+        login: str,
+        role: str,
+        can_view_path: Optional[bool] = None,
+        notify: Optional[bool] = None,
+        fields: Iterable[str] = None,
     ) -> 'Collaboration':
         """Collaborate user onto a Box item with the user login.
 
@@ -457,10 +349,10 @@ class Item(BaseItem):
 
     @api_call
     def get_collaborations(
-            self,
-            limit: Optional[int] = None,
-            marker: Optional[str] = None,
-            fields: Iterable[str] = None
+        self,
+        limit: Optional[int] = None,
+        marker: Optional[str] = None,
+        fields: Iterable[str] = None,
     ) -> 'BoxObjectCollection':
         """
         Get the entries in the collaboration.
@@ -497,8 +389,7 @@ class Item(BaseItem):
             'Box__Security__Classification__Key': classification,
         }
         metadata_classification = self.metadata(
-            scope='enterprise',
-            template=self._classification_template_key
+            scope='enterprise', template=self._classification_template_key
         ).create(classification_metadata)
         return metadata_classification['Box__Security__Classification__Key']
 
@@ -511,7 +402,9 @@ class Item(BaseItem):
         :return:
             The classification updated on the :class:`File` or :class:`Folder.
         """
-        classification_metadata = self.metadata('enterprise', self._classification_template_key)
+        classification_metadata = self.metadata(
+            'enterprise', self._classification_template_key
+        )
         updates = classification_metadata.start_update()
         updates.add('/Box__Security__Classification__Key', classification)
         metadata_classification = classification_metadata.update(updates)
@@ -531,8 +424,7 @@ class Item(BaseItem):
             'Box__Security__Classification__Key': classification,
         }
         return self.metadata(
-            scope='enterprise',
-            template=self._classification_template_key
+            scope='enterprise', template=self._classification_template_key
         ).set(metadata=classification_metadata)['Box__Security__Classification__Key']
 
     def get_classification(self) -> Optional[str]:
@@ -543,7 +435,9 @@ class Item(BaseItem):
             The classification on the :class:`File` or :class:`Folder.
         """
         try:
-            classification = self.metadata('enterprise', self._classification_template_key).get()
+            classification = self.metadata(
+                'enterprise', self._classification_template_key
+            ).get()
         except BoxAPIException as err:
             if err.status == 404 and err.code == "instance_not_found":
                 return None
