@@ -4,8 +4,9 @@ import math
 from functools import partial
 from logging import getLogger
 from numbers import Number
-from typing import TYPE_CHECKING, Optional, Any, Type, Callable
+from typing import TYPE_CHECKING, Optional, Any, Type, Callable, Set
 
+from requests.exceptions import RequestException
 from boxsdk.exception import BoxException
 from .box_request import BoxRequest as _BoxRequest
 from .box_response import BoxResponse as _BoxResponse
@@ -28,19 +29,21 @@ class Session:
     _retry_randomization_factor = 0.5
     _retry_base_interval = 1
     _JWT_GRANT_TYPE = 'urn:ietf:params:oauth:grant-type:jwt-bearer'
+    _CCG_GRANT_TYPE = 'client_credentials'
 
     """
     Box API session. Provides automatic retry of failed requests.
     """
+
     def __init__(
-            self,
-            network_layer: 'Network' = None,
-            default_headers: Optional['dict'] = None,
-            translator: Translator = None,
-            default_network_request_kwargs: Optional['dict'] = None,
-            api_config: API = None,
-            client_config: Client = None,
-            proxy_config: Optional[Proxy] = None,
+        self,
+        network_layer: 'Network' = None,
+        default_headers: Optional['dict'] = None,
+        translator: Translator = None,
+        default_network_request_kwargs: Optional['dict'] = None,
+        api_config: API = None,
+        client_config: Client = None,
+        proxy_config: Optional[Proxy] = None,
     ):
         """
         :param network_layer:
@@ -113,6 +116,8 @@ class Session:
         """
         if 'expect_json_response' not in kwargs:
             kwargs['expect_json_response'] = False
+        if 'skip_retry_codes' not in kwargs:
+            kwargs['skip_retry_codes'] = {202}
         return self.request('DELETE', url, **kwargs)
 
     def options(self, url: str, **kwargs: Any) -> '_BoxResponse':
@@ -172,21 +177,22 @@ class Session:
         :param args:
             Additional parts of the endpoint URL.
         """
-        # pylint:disable=no-self-use
         url = [f'{self._api_config.BASE_API_URL}/{endpoint}']
         url.extend([f'/{x}' for x in args])
         return ''.join(url)
 
     def get_constructor_kwargs(self) -> dict:
-        return dict(
-            network_layer=self._network_layer,
-            translator=self._translator,
-            default_network_request_kwargs=self._default_network_request_kwargs.copy(),
-            api_config=self._api_config,
-            client_config=self._client_config,
-            proxy_config=self._proxy_config,
-            default_headers=self._default_headers.copy(),
-        )
+        return {
+            'network_layer': self._network_layer,
+            'translator': self._translator,
+            'default_network_request_kwargs': (
+                self._default_network_request_kwargs.copy()
+            ),
+            'api_config': self._api_config,
+            'client_config': self._client_config,
+            'proxy_config': self._proxy_config,
+            'default_headers': self._default_headers.copy(),
+        }
 
     def as_user(self, user: 'User') -> 'Session':
         """
@@ -199,7 +205,9 @@ class Session:
         kwargs['default_headers']['As-User'] = user.object_id
         return self.__class__(**kwargs)
 
-    def with_shared_link(self, shared_link: str, shared_link_password: str = None) -> 'Session':
+    def with_shared_link(
+        self, shared_link: str, shared_link_password: str = None
+    ) -> 'Session':
         """
         Returns a new session object with default headers set up to make requests using the shared link for auth.
 
@@ -209,10 +217,14 @@ class Session:
             The password for the shared link.
         """
         kwargs = self.get_constructor_kwargs()
-        kwargs['default_headers'].update(get_shared_link_header(shared_link, shared_link_password))
+        kwargs['default_headers'].update(
+            get_shared_link_header(shared_link, shared_link_password)
+        )
         return self.__class__(**kwargs)
 
-    def with_default_network_request_kwargs(self, extra_network_parameters: dict) -> 'Session':
+    def with_default_network_request_kwargs(
+        self, extra_network_parameters: dict
+    ) -> 'Session':
         kwargs = self.get_constructor_kwargs()
         kwargs['default_network_request_kwargs'].update(extra_network_parameters)
         return self.__class__(**kwargs)
@@ -220,7 +232,9 @@ class Session:
     # We updated our retry strategy to use exponential backoff instead of the header returned from the API response.
     # This is something we can remove in latter major bumps.
     # pylint: disable=unused-argument
-    def get_retry_after_time(self, attempt_number: int, retry_after_header: Optional[str]) -> Number:
+    def get_retry_after_time(
+        self, attempt_number: int, retry_after_header: Optional[str]
+    ) -> Number:
         """
         Get the amount of time to wait before retrying the API request, using the attempt number that failed to
         calculate the retry time for the next retry attempt.
@@ -241,12 +255,18 @@ class Session:
                 pass
         min_randomization = 1 - self._retry_randomization_factor
         max_randomization = 1 + self._retry_randomization_factor
-        randomization = (random.uniform(0, 1) * (max_randomization - min_randomization)) + min_randomization
+        randomization = (
+            random.uniform(0, 1) * (max_randomization - min_randomization)
+        ) + min_randomization
         exponential = math.pow(2, attempt_number)
         return exponential * self._retry_base_interval * randomization
 
     @staticmethod
-    def _raise_on_unsuccessful_request(network_response: 'NetworkResponse', request: '_BoxRequest') -> None:
+    def _raise_on_unsuccessful_request(
+        network_response: 'NetworkResponse',
+        request: '_BoxRequest',
+        raised_exception: Exception,
+    ) -> None:
         """
         Raise an exception if the request was unsuccessful.
 
@@ -255,6 +275,9 @@ class Session:
         :param request:
             The API request that could be unsuccessful.
         """
+        if network_response is None:
+            raise raised_exception
+
         if not network_response.ok:
             response_json = {}
             try:
@@ -264,15 +287,18 @@ class Session:
             raise BoxAPIException(
                 status=network_response.status_code,
                 headers=network_response.headers,
-                code=response_json.get('code', None) or response_json.get('error', None),
-                message=response_json.get('message', None) or response_json.get('error_description', None),
+                code=response_json.get('code', None)
+                or response_json.get('error', None),
+                message=response_json.get('message', None)
+                or response_json.get('error_description', None),
                 request_id=response_json.get('request_id', None),
                 url=request.url,
                 method=request.method,
                 context_info=response_json.get('context_info', None),
-                network_response=network_response
+                network_response=network_response,
             )
-        if request.expect_json_response and not is_json_response(network_response):
+
+        if not Session._is_json_response_if_expected(network_response, request):
             raise BoxAPIException(
                 status=network_response.status_code,
                 headers=network_response.headers,
@@ -282,14 +308,28 @@ class Session:
                 network_response=network_response,
             )
 
+    @staticmethod
+    def _is_json_response_if_expected(
+        network_response: 'NetworkResponse', request: '_BoxRequest'
+    ) -> bool:
+        """
+        Validate that the response is json if the request expects json response.
+
+        :param network_response:
+            The network response which is being tested for success.
+        :param request:
+            The API request that could be unsuccessful.
+        """
+        return not request.expect_json_response or is_json_response(network_response)
+
     def _prepare_and_send_request(
-            self,
-            method: str,
-            url: str,
-            headers: dict = None,
-            auto_session_renewal: bool = True,
-            expect_json_response: bool = True,
-            **kwargs: Any
+        self,
+        method: str,
+        url: str,
+        headers: dict = None,
+        auto_session_renewal: bool = True,
+        expect_json_response: bool = True,
+        **kwargs: Any,
     ) -> 'NetworkResponse':
         """
         Prepare a request to be sent to the Box API.
@@ -301,14 +341,16 @@ class Session:
         :param headers:
             Headers to include with the request.
         :param auto_session_renewal:
-            Whether or not to automatically renew the session if the request fails due to an expired access token.
+            Whether to automatically renew the session if the request fails due to an expired access token.
         :param expect_json_response:
-            Whether or not the response content should be json.
+            Whether the response content should be json.
         """
         files = kwargs.get('files')
         kwargs['file_stream_positions'] = None
         if files:
-            kwargs['file_stream_positions'] = {name: file_tuple[1].tell() for name, file_tuple in files.items()}
+            kwargs['file_stream_positions'] = {
+                name: file_tuple[1].tell() for name, file_tuple in files.items()
+            }
         attempt_number = 0
         request_headers = self._get_request_headers()
         request_headers.update(headers or {})
@@ -322,28 +364,59 @@ class Session:
         )
 
         skip_retry_codes = kwargs.pop('skip_retry_codes', set())
-        network_response = self._send_request(request, **kwargs)
+
+        raised_exception = None
+        try:
+            network_response = self._send_request(request, **kwargs)
+            reauthentication_needed = network_response.status_code == 401
+        except RequestException as request_exc:
+            raised_exception = request_exc
+            network_response = None
+            if 'EOF occurred in violation of protocol' in str(request_exc):
+                reauthentication_needed = True
+            elif any(
+                text in str(request_exc)
+                for text in [
+                    'Connection aborted',
+                    'Connection broken',
+                    'Connection reset',
+                ]
+            ):
+                reauthentication_needed = False
+            else:
+                raise
 
         while True:
-            retry = self._get_retry_request_callable(network_response, attempt_number, request, **kwargs)
+            retry = self._get_retry_request_callable(
+                network_response,
+                attempt_number,
+                request,
+                skip_retry_codes,
+                reauthentication_needed,
+                **kwargs,
+            )
 
-            if retry is None or attempt_number >= API.MAX_RETRY_ATTEMPTS or network_response.status_code in skip_retry_codes:
+            if retry is None or attempt_number >= API.MAX_RETRY_ATTEMPTS:
+                if network_response is None:
+                    raise raised_exception
                 break
 
             attempt_number += 1
             self._logger.debug('Retrying request')
             network_response = retry(request, **kwargs)
 
-        self._raise_on_unsuccessful_request(network_response, request)
+        self._raise_on_unsuccessful_request(network_response, request, raised_exception)
 
         return network_response
 
     def _get_retry_request_callable(
-            self,
-            network_response: 'NetworkResponse',
-            attempt_number: int,
-            request: '_BoxRequest',
-            **kwargs: Any
+        self,
+        network_response: Optional['NetworkResponse'],
+        attempt_number: int,
+        request: '_BoxRequest',
+        skip_retry_codes: Set[int],
+        session_renewal_needed: bool = False,
+        **kwargs: Any,
     ) -> Optional[Callable]:
         """
         Get a callable that retries a request for certain types of failure.
@@ -364,6 +437,34 @@ class Session:
             Callable that, when called, will retry the request. Takes the same parameters as :meth:`_send_request`.
         """
         # pylint:disable=unused-argument
+        # pylint:disable=line-too-long
+        if network_response is None or (
+            network_response.ok
+            and request.method == 'GET'
+            and not self._is_json_response_if_expected(network_response, request)
+        ):
+            return partial(
+                self._network_layer.retry_after,
+                self.get_retry_after_time(attempt_number, None),
+                self._send_request,
+            )
+        code = network_response.status_code
+
+        if (
+            (code in (202, 429) or code >= 500)
+            and code not in skip_retry_codes
+            and not self._is_server_auth_type(kwargs)
+        ):
+            return partial(
+                self._network_layer.retry_after,
+                self.get_retry_after_time(
+                    attempt_number, network_response.headers.get('Retry-After', None)
+                ),
+                self._send_request,
+            )
+        return None
+
+    def _is_server_auth_type(self, kwargs: dict) -> bool:
         data = kwargs.get('data', {})
         grant_type = None
         try:
@@ -371,14 +472,7 @@ class Session:
                 grant_type = data['grant_type']
         except TypeError:
             pass
-        code = network_response.status_code
-        if (code in (202, 429) or code >= 500) and grant_type != self._JWT_GRANT_TYPE:
-            return partial(
-                self._network_layer.retry_after,
-                self.get_retry_after_time(attempt_number, network_response.headers.get('Retry-After', None)),
-                self._send_request,
-            )
-        return None
+        return grant_type in (self._JWT_GRANT_TYPE, self._CCG_GRANT_TYPE)
 
     def _get_request_headers(self) -> dict:
         return self._default_headers.copy()
@@ -393,16 +487,22 @@ class Session:
         proxy = {}
         if self._proxy_config.URL is None:
             return None
-        if self._proxy_config.AUTH and {'user', 'password'} <= set(self._proxy_config.AUTH):
+        if self._proxy_config.AUTH and {'user', 'password'} <= set(
+            self._proxy_config.AUTH
+        ):
             host = self._proxy_config.URL
             address = host.split('//')[1]
-            proxy_string = f'http://{self._proxy_config.AUTH.get("user", None)}:' \
-                           f'{self._proxy_config.AUTH.get("password", None)}@{address}'
+            proxy_string = (
+                f'http://{self._proxy_config.AUTH.get("user", None)}:'
+                f'{self._proxy_config.AUTH.get("password", None)}@{address}'
+            )
         elif self._proxy_config.AUTH is None:
             proxy_string = self._proxy_config.URL
         else:
-            raise BoxException("The proxy auth dict you provided does not match pattern "
-                               "{'user': 'example_user', 'password': 'example_password'}")
+            raise BoxException(
+                "The proxy auth dict you provided does not match pattern "
+                "{'user': 'example_user', 'password': 'example_password'}"
+            )
         proxy['http'] = proxy_string
         proxy['https'] = proxy['http']
 
@@ -417,7 +517,11 @@ class Session:
         """
         # Reset stream positions to what they were when the request was made so the same data is sent even if this
         # is a retried attempt.
-        files, file_stream_positions = kwargs.get('files'), kwargs.pop('file_stream_positions')
+        files, file_stream_positions, stream_file_content = (
+            kwargs.get('files'),
+            kwargs.pop('file_stream_positions'),
+            kwargs.pop('stream_file_content', True),
+        )
         request_kwargs = self._default_network_request_kwargs.copy()
         request_kwargs.update(kwargs)
         proxy_dict = self._prepare_proxy()
@@ -426,19 +530,22 @@ class Session:
         if files and file_stream_positions:
             for name, position in file_stream_positions.items():
                 files[name][1].seek(position)
-            data = request_kwargs.pop('data', {})
-            multipart_stream = MultipartStream(data, files)
-            request_kwargs['data'] = multipart_stream
-            del request_kwargs['files']
-            request.headers['Content-Type'] = multipart_stream.content_type
+            if stream_file_content:
+                data = request_kwargs.pop('data', {})
+                multipart_stream = MultipartStream(data, files)
+                request_kwargs['data'] = multipart_stream
+                del request_kwargs['files']
+                request.headers['Content-Type'] = multipart_stream.content_type
+        request.access_token = request_kwargs.pop('access_token', None)
 
         # send the request
         network_response = self._network_layer.request(
             request.method,
             request.url,
-            access_token=request_kwargs.pop('access_token', None),
+            access_token=request.access_token,
             headers=request.headers,
-            **request_kwargs
+            log_response_content=request.expect_json_response,
+            **request_kwargs,
         )
 
         return network_response
@@ -475,11 +582,13 @@ class AuthorizedSession(Session):
         return new_access_token
 
     def _get_retry_request_callable(
-            self,
-            network_response: 'NetworkResponse',
-            attempt_number: int,
-            request: '_BoxRequest',
-            **kwargs: Any
+        self,
+        network_response: Optional['NetworkResponse'],
+        attempt_number: int,
+        request: '_BoxRequest',
+        skip_retry_codes: Set[int],
+        session_renewal_needed: bool = False,
+        **kwargs: Any,
     ) -> Callable:
         """
         Get a callable that retries a request for certain types of failure.
@@ -497,16 +606,18 @@ class AuthorizedSession(Session):
         :return:
             Callable that, when called, will retry the request. Takes the same parameters as :meth:`_send_request`.
         """
-        code = network_response.status_code
-        if code == 401 and request.auto_session_renewal:
-            self._renew_session(network_response.access_token_used)
+        if request.auto_session_renewal and session_renewal_needed:
+            self._renew_session(request.access_token)
             request.auto_session_renewal = False
             return self._send_request
+
         return super()._get_retry_request_callable(
             network_response,
             attempt_number,
             request,
-            **kwargs
+            skip_retry_codes,
+            session_renewal_needed,
+            **kwargs,
         )
 
     def _send_request(self, request: '_BoxRequest', **kwargs: Any) -> 'NetworkResponse':
